/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.actionscript-facile.com
  Portions created by the Initial Developer are Copyright (C) 2010
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */
package cours
{

	import flash.geom.Vector3D;
	
	import away3d.cameras.Camera3D;
	import away3d.containers.ObjectContainer3D;
	import away3d.containers.Scene3D;
	import away3d.containers.View3D;
	import away3d.core.utils.Cast;
	import away3d.lights.DirectionalLight3D;
	import away3d.materials.PhongBitmapMaterial;
	import away3d.primitives.Sphere;

	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;

	
	
	
	public class Globe extends Sprite
	{
		// texture pour le globe
		[Embed(source="../bin/as3-formation-logo.png")]
		public static var EarthImage:Class;
		    	
		// variables du moteur
		private var scene:Scene3D;
		private var view:View3D;
		
		// objets des textures
		private var material:PhongBitmapMaterial;
		
		// objets de la scène
		private var sphere:Sphere;
		private var spherecontainer:ObjectContainer3D;
		
		// variables pour la navigation
		private var move:Boolean = false;
		private var lastMouseX:Number;
		private var lastMouseY:Number;
		private var lastRotationX:Number;
		private var lastRotationY:Number;
		
		
		/**
		 * Constructor
		 */
		public function Globe() 
		{
		}
		
		
		/**
		 * Global init.
		 * 
		 */
		public function init():void
		{
			initEngine();
			initMaterials();
			initObjects();
			initLights();
			initListeners();
		}
		
		/**
		 * Init du moteur.
		 * 
		 */
		private function initEngine():void
		{
			scene = new Scene3D();
			
			//camera = new Camera3D({z:-1000});
			var camera:Camera3D = new Camera3D();
			camera.z = -300;// zoom arrière
			
			//view = new View3D({scene:scene, camera:camera});
			view = new View3D();
			view.scene= scene;
			view.camera = camera;
			// ajout l'url de la source dans le menu contextuel
			view.addSourceURL("http://www.actionscript-facile.com");
			addChild(view);
		}
		
		/**
		 * Init les textures.
		 * 
		 */
		private function initMaterials():void
		{
			//material = new PhongBitmapMaterial(Cast.bitmap(EarthImage), {specular:0x1A1A1A, shininess:10});
			material = new PhongBitmapMaterial(Cast.bitmap(EarthImage));
			material.specular = 0x006f77;
			material.shininess = 10;
		}
		
		/**
		 * Init des objets de la scene
		 */
		private function initObjects():void
		{
			//sphere = new Sphere({material:material, radius:200, segmentsW:40, segmentsH:20});
			sphere = new Sphere();
			sphere.material = material;// affectation de la texture à la sphère
			sphere.radius = 60;// diamètre de la sphère
			sphere.segmentsW = 30;// nombre de segments ou lignes composant la sphère partie largeur
			sphere.segmentsH = 15;// nombre de segments ou lignes composant la sphère partie hauteur
			
			spherecontainer = new ObjectContainer3D(sphere);
			scene.addChild(spherecontainer);
		}
		
		/**
		 * Init des lumières.
		 * 
		 */
		private function initLights():void
		{
			//light = new DirectionalLight3D({x:1, y:1, z:-1, ambient:0.2});
			var light:DirectionalLight3D = new DirectionalLight3D();
			light.direction = new Vector3D(-1, -1, 1);
			light.ambient = 0.2;
			
			scene.addLight(light);
		}
		
		/**
		 * Init des écouteurs.
		 * 
		 */
		private function initListeners():void
		{
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(Event.RESIZE, onResize);
			onResize();
		}
		
		/**
		 * Le rendu et la navigation
		 * Mise à jour sur chaque frame.
		 * 
		 */
		private function onEnterFrame(e:Event):void
		{
			sphere.rotationY += 1;
			
			if (move) {
				spherecontainer.rotationX = (mouseY - lastMouseY)/2 + lastRotationX;
				if (spherecontainer.rotationX > 90)
					spherecontainer.rotationX = 90;
				if (spherecontainer.rotationX < -90)
					spherecontainer.rotationX = -90;
				sphere.rotationY = (lastMouseX - mouseX)/2 + lastRotationY;
			}
			
			view.render();
		}
		
		/**
		 * Ecouteur de la souris pour la navigation
		 * Le mouvement de la sphère.
		 * 
		 */
		private function onMouseDown(e:MouseEvent):void
		{
			lastRotationX = spherecontainer.rotationX;
			lastRotationY = sphere.rotationY;
			lastMouseX = mouseX;
			lastMouseY = mouseY;
			move = true;
			stage.addEventListener(Event.MOUSE_LEAVE, onStageMouseLeave);
		}
		
		/**
		 * Ecouteur de la souris pour la navigation
		 * Le mouvement de la sphère.
		 * 
		 */
		private function onMouseUp(e:MouseEvent):void
		{
			move = false;
			stage.removeEventListener(Event.MOUSE_LEAVE, onStageMouseLeave);    
		}
        
		/**
		 * Ecouteur de la souris (sur le stage) pour la navigation.
		 * 
		 * 
		 */
        private function onStageMouseLeave(event:Event):void
        {
        	move = false;
        	stage.removeEventListener(Event.MOUSE_LEAVE, onStageMouseLeave);     
        }
		
		
		/**
		 * Ecouteur sur le stage et redimensionnement.
		 * 
		 */
		private function onResize(event:Event = null):void
		{
			view.x = stage.stageWidth / 2;
            view.y = stage.stageHeight / 2;
		}
	}
}